--[[ AutoTarget: Automatically selects matching targets.
    Written by Vincent of Blackhand, (copyright 2005 by D.A.Down)

    Version history:
    0.7 - Initial public release.

]]--

local FCr = "|cffff4040";
local MinLevel,MaxLevel,Level,Names,Opt,AT_On,Debug = 0,0,0,{};
local function print(msg) SELECTED_CHAT_FRAME:AddMessage("AT: "..msg, 0,1,1); end
local function UnitIsNeutral(unit) return UnitReaction(unit,'player')==4; end
local function UnitIsElite(unit) return UnitClassification(unit)=='elite'; end
local function UnitIsCritter(unit) return UnitCreatureType(unit)=='Critter'; end
local function UnitIsDeadly(unit) return Level<0; end
AutoTarget_Option = {};

function AutoTarget_Initialize()
    UIPanelWindows["AT_Config"] = {area="left", pushable=11};
    -- add our chat commands
    SlashCmdList["AUTOTARGET"] = AutoTarget_command;
    SLASH_AUTOTARGET1 = "/at";
    -- setup option window
    Opt = { Enemy   = {B=AT_EnemyButton,   F=UnitIsEnemy},
            Neutral = {B=AT_NeutralButton, F=UnitIsNeutral},
            Friend  = {B=AT_FriendButton,  F=UnitIsFriend},
            Elite   = {B=AT_EliteButton,   F=UnitIsElite,   X=AT_EliteExcl},
            Player  = {B=AT_PlayerButton,  F=UnitIsPlayer,  X=AT_PlayerExcl},
            PvP     = {B=AT_PvPButton,     F=UnitIsPVP,     X=AT_PvPExcl},
            Critter = {B=AT_CritterButton, F=UnitIsCritter, X=AT_CritterExcl},
            Charmed = {B=AT_CharmedButton, F=UnitIsCharmed},
            Civilian= {B=AT_CivilianButton,F=UnitIsCivilian},
            Trivial = {B=AT_TrivialButton, F=UnitIsTrivial},
            Deadly  = {B=AT_DeadlyButton,  F=UnitIsDeadly},
          };
    AT_OnButton.Tip = AT_ON_TIP;
    for name,data in Opt do
      data.B.Tip = getglobal('AT_'..strupper(name)..'_TIP');
      if(data.X) then
        data.X.Tie = data.B;
        data.B.Tie = data.X;
      end
    end
end

-- Handle mouse-overs
function AutoTarget_MouseOver(unit)
    if(not AT_On) then return; end
    if(UnitExists('target') and not UnitIsDead('target')) then return; end
    local fail = AutoTarget_CheckUnit(unit);
    if(Debug and fail) then
      print(UnitName(unit)..' skipped on '..fail);
    end
end

function AutoTarget_CheckUnit(unit)
    if(UnitIsDead(unit)) then return 'Dead'; end
    Level = UnitLevel(unit);
    if(Level<MinLevel and Level>0) then return 'MinLevel'; end
    if(MaxLevel>0 and Level>MaxLevel) then return 'MaxLevel'; end
    local set,match;
    for name,data in Opt do
      set,match = AutoTarget_Option[name], data.F(unit,"player");
      if(match and not set or set==2 and not match) then return name; end
    end
    if(getn(Names)>0) then
      local match,uname = false,strlower(UnitName(unit));
      for ix,name in Names do
        if(strfind(uname,name)) then match = true; break; end
      end
      if(not match) then return 'Name'; end
    end
    PlaySound("igCharacterInfoTab");
    TargetUnit(unit);
end

-- Handle our configuration setting
function AutoTarget_Toggle_Config()
    if(AT_Config:IsVisible()) then HideUIPanel(AT_Config);
    else SetCenterFrame(AT_Config,true); ShowUIPanel(AT_Config); end
end

local function SetName(field)
    field:SetText(field.old or '');
end

function AutoTarget_Show()
    AT_OnButton:SetChecked(AT_On);
    AT_MinLevel:SetText(MinLevel>0 and MinLevel or '');
    AT_MaxLevel:SetText(MaxLevel>0 and MaxLevel or '');
    SetName(AT_Name1Text);
    SetName(AT_Name2Text);
    SetName(AT_Name3Text);
    for name,data in Opt do
      data.B:SetChecked(AutoTarget_Option[name]);
      if(data.X) then
        data.X:SetChecked(AutoTarget_Option[name]==2);
      end
    end
end

local function GetName(field)
    local name = field:GetText();
    field.old = name;
    if(name~='') then tinsert(Names,strlower(name)); end
end

function AutoTarget_Config()
    AT_On = AT_OnButton:GetChecked();
    MinLevel = AT_MinLevel:GetNumber();
    MaxLevel = AT_MaxLevel:GetNumber();
    if(MaxLevel>0 and MaxLevel<MinLevel) then
      PlaySound("igQuestLogAbandonQuest");
      AT_MaxLevel:SetFocus();
      return;
    end
    Names = {};
    GetName(AT_Name1Text);
    GetName(AT_Name2Text);
    GetName(AT_Name3Text);
    for name,data in Opt do
      AutoTarget_Option[name] = data.B:GetChecked();
      if(data.X and data.X:GetChecked()) then
        AutoTarget_Option[name] = 2;
      end
    end
    HideUIPanel(AT_Config);
end

function TS_OnClick(field)
    if(UnitExists('target')) then
      getglobal(field.."Text"):SetText(UnitName('target'));
    end
end

function AutoTarget_Clear()
    AT_MinLevel:SetText('');
    AT_MaxLevel:SetText('');
    AT_Name1Text:SetText('');
    AT_Name2Text:SetText('');
    AT_Name3Text:SetText('');
    for name,data in Opt do
      data.B:SetChecked();
    end
end

-- Handle our chat command
function AutoTarget_command(msg)
    if(msg=='config') then AutoTarget_Toggle_Config();
    elseif(msg=='') then
      AT_On = not AT_On;
      print('AutoTarget '..(AT_On and AT_ON or AT_OFF));
    elseif(msg=='debug') then
      Debug = not Debug;
      print(AT_DEBUG..(Debug and AT_ON or AT_OFF));
    else print(AT_UNKNOWN..msg); end
end
